## set up matrix with harmonic seasonal regressors
## based on x taking values 1:freq
harmonic <- function(x, order = 2, freq = 365) {
  x <- x/freq
  order <- round(order)
  stopifnot(order <= freq/2)
  rval <- outer(2 * pi * x, 1:order)
  rval <- cbind(apply(rval, 2, cos), apply(rval, 2, sin))
  colnames(rval) <- if (order == 1) {
    c("cos", "sin")
  } else {
    c(paste("cos", 1:order, sep = ""), paste("sin", 1:order, sep = ""))
  }
  if ((2 * order) == freq) rval <- rval[, -(2 * order)]
  return(rval)
}

## compute day in year (omitting Feb 29th, if any)
yday365 <- function(x) {
   x <- as.POSIXlt(x)
   mdays <- c(31L, 28L, 31L, 30L, 31L, 30L, 31L, 31L, 30L, 31L, 30L, 31L)
   cumsum(c(0L, mdays))[1L + x$mon] + x$mday
}

## set up data frame with time series response as well as several
## auxiliary quantities (categorical variables, seasonal regressors, weekend, etc.)
tspp <- function(x,
  breaks = c(0.01, 0.99, 4.99), labels = c("none", "low", "medium", "high"),
  origin = 1970, order = 2)
{
  stopifnot(require("zoo"))
  
  ## only use first column of data
  x <- x[, 1L]

  ## time scale
  tx <- time(x)
  stopifnot(inherits(tx, "Date"))

  ## convenience functions for processing "Date" objects
  is_weekday <- function(z) {
    if(inherits(z, "zoo")) z <- time(z)
    z <- as.POSIXlt(z)$wday
    z > 0L & z < 6L
  }
  is_weekend <- function(z) {
    if(inherits(z, "zoo")) z <- time(z)
    z <- as.POSIXlt(z)$wday
    z < 1L | z > 5L
  }
  is_Feb29 <- function(z) {
    if(inherits(z, "zoo")) z <- time(z)
    z <- as.POSIXlt(z)
    z$mon == 1L & z$mday == 29L
  }

  ## remove Feb 29
  x <- x[!is_Feb29(tx)]
  tx <- time(x)
  
  ## set up return value
  rval <- data.frame(
    raw = as.numeric(x),                                                             ## original zoo series
    cens = ifelse(coredata(x) >= 0, coredata(x), 0),                                 ## censored at zero
    bin = factor(coredata(x) > 0, levels = c(FALSE, TRUE), labels = c("no", "yes")), ## binary: greater than zero?
    cat = cut(coredata(x), breaks = c(-Inf, breaks, Inf), labels = labels),          ## ordered categorical
    trend = 1900 - origin + as.POSIXlt(tx)$year + (yday365(tx) - 1)/365,             ## years since origin
    month = factor(as.POSIXlt(tx)$mon + 1, levels = 1:12, labels = month.abb),       ## month of year
    year = factor(1900 + as.POSIXlt(tx)$year),                                       ## year
    day = yday365(tx),                                                               ## day of year (omitting Feb 29)
    weekend = factor(is_weekend(tx), labels = c("no", "yes"))                        ## weekend?
  )

  rval$raw <- x
  rval$harmon <- harmonic(rval$day)                                                  ## harmonic regressor matrix
  return(rval)
}

extract_effects <- function(object = homstart_model, metainfo = homstart)
{
  stopifnot(require("R2BayesX"))

  ## extract all coefficients (point estimates only)
  cf <- coef(object)
  ncf <- rownames(cf)
  cf <- cf[, 1L]

  ## extract seasonal effects for each station (plus long/lat)
  param <- cbind(
    "long" = round(metainfo$long, 4),
    "lat" = round(metainfo$lat, 4),
    "weekend" = cf[grep("weekend", ncf, fixed = TRUE)],
    "cos1" = cf[grep("cos1", ncf, fixed = TRUE)],
    "cos2" = cf[grep("cos2", ncf, fixed = TRUE)],
    "sin1" = cf[grep("sin1", ncf, fixed = TRUE)],
    "sin2" = cf[grep("sin2", ncf, fixed = TRUE)]
  )
  param <- cbind(param,
    "amplitude1" = sqrt(param[, 1L + 3L]^2 + param[, 3L + 3L]^2),
    "amplitude2" = sqrt(param[, 2L + 3L]^2 + param[, 4L + 3L]^2),
    "phase1" = param[, 3L + 3L] / param[, 1L + 3L],
    "phase2" = param[, 4L + 3L] / param[, 2L + 3L]
  )
  rownames(param) <- as.character(metainfo$id)
  param <- param[order(param[, "lat"]), ]
  param <- param[order(param[, "long"]), ]
  
  ## fitted seasonal effect for each station
  day <- 1:365
  for(i in 1:nrow(param))
    day <- cbind(day, harmonic(1:365) %*% param[i, (3:6 + 1)])
  day <- as.data.frame(day)
  colnames(day) <- c("day", rownames(param))

  ## all effects
  rval <- list(
    cutoffs = cf[1:3],
    param = as.data.frame(param),
    day = day,
    spatial = fitted(object, term = "sx(long,lat)"),
    trend = fitted(object, term = "sx(trend)")
  )
  class(rval$param) <- c("coef.bayesx", "data.frame")
  return(rval)
}

## function to plot interpolated map effects
interpolmap <- function(x, y = NULL, z = NULL, map, select = 3, grid = 300,
  col = NULL, ncol = 99, symmetric = TRUE, breaks = NULL, range = NULL, lrange = NULL,
  swap = TRUE, p.pch = 16, p.cex = 1, legend = TRUE, linear = FALSE, extrap = FALSE,
  bgcol = "transparent", stations = TRUE, s.col = "black", s.pch = 21,
  s.cex = 3, ...)
{
  ## required packages
  stopifnot(require("maptools"), require("R2BayesX"))
  gpclibPermit()

  ## convenience function for deleting arguments from lists
  delete.args <- function(fun = NULL, args = NULL, not = NULL) {
    nf <- names(formals(fun))
    na <- names(args)
    for(elmt in na)
      if(!elmt %in% nf) {
        if(!is.null(not)) {
          if(!elmt %in% not)
            args[elmt] <- NULL
        } else args[elmt] <- NULL
      }
  
    return(args)
  }

  if(!is.null(y) && grepl("Spatial", class(y), ignore.case = TRUE)) map <- y

  if(grepl(".bayesx", class(x)[1L], fixed = TRUE)) {
    if(is.character(select))
      select <- grep(select, colnames(x), ignore.case = TRUE)[1L]
    if((x[1L, 1L] == 0) && (x[1L, 2L] == 0))
      x <- x[2L:nrow(x),]
    y <- x[, 2L]
    z <- x[, select]
    x <- x[, 1L]
  }

  mapbox <- bbox(map)
  ico <- akima::interp(x = x, y = y, z = z,
    xo = seq(mapbox[1L, 1L], mapbox[1L, 2L], length = grid),
    yo = seq(mapbox[2L, 1L], mapbox[2L, 2L], length = grid),
    duplicate = "strip", linear = linear, extrap = extrap)

  yco <- rep(ico$y, each = length(ico$x))
  xco <- rep(ico$x, length(ico$y))

  ob <- unionSpatialPolygons(map, rep(1L, length = length(map)))
  oco <- slot(slot(slot(ob, "polygons")[[1]], "Polygons")[[1]], "coords")
  pip <- point.in.polygon(xco, yco, oco[, 1L], oco[, 2L], mode.checked = FALSE) < 1L

  if(is.null(col))
    col <- colorspace::diverge_hcl
  cvals <- as.numeric(ico$z)
  cvals[cvals < min(z)] <- min(z)
  cvals[cvals > max(z)] <- max(z)
  pal <- R2BayesX:::make_pal(col = col, ncol = 99, data = cvals, swap = swap,
    range = range, symmetric = symmetric, breaks = breaks)
  colors <- pal$map(cvals)
  colors[pip] <- NA

  if(extrap)
    plot(map)
  else
    plot(map, col = bgcol)
  points(SpatialPoints(cbind(xco, yco)), col = colors, pch = p.pch, cex = p.cex)
  plot(map, add = TRUE)

  if(legend) {
    args <- list(...)
    if(is.null(args$pos))
      args$pos <- c(0.04, 0.88)
    args$xlim <- par()$usr[1L:2L]
    args$ylim <- par()$usr[3L:4L]
    args$x <- cvals
    args$color <- col
    args$ncol <- ncol
    args$symmetric <- symmetric
    args$swap <- swap
    args$range <- range
    args$breaks <- breaks
    args$plot <- FALSE
    args$add <- TRUE
    if(is.null(lrange))
      args$lrange <- args$range
    else
      args$lrange <- lrange
    if(is.null(args$width))
      args$width <- 0.5
    if(is.null(args$height))
      args$height <- 0.25
    if(is.null(args$length.ticks))
      args$length.ticks <- 3
    if(is.null(args$distance.labels))
      args$distance.labels <- 1.5
    if(is.null(args$cex.labels))
      args$cex.labels <- 1.2
    do.call(R2BayesX::colorlegend, delete.args(R2BayesX::colorlegend, args))
  }
  if(stations) {
    points(SpatialPoints(cbind(x, y),
      proj4string = attr(map, "proj4string")),
      col = s.col, pch = s.pch, cex = s.cex,
      bg = pal$map(z))
  }

  invisible(pal)
}

probeffects <- function(effects = homstart_effects, metainfo = homstart,
  trend = FALSE, harmon = NULL, spatial = TRUE, weekend = FALSE, stations = NULL)
{
  ## seasonal effect
  e_ha <- effects$param[, 4:7]

  ## spatial effects
  e_sp <- effects$spatial
  e_we <- effects$param$weekend
  
  ## contributions to linear predictor  
  if(is.null(harmon)) {
    trend <- FALSE
    p_ha <- 0
  } else {
    harmon <- as.Date(harmon)
    p_ha <- as.matrix(e_ha) %*% harmonic(c(0, yday365(harmon)))[-1L,]
  }  
  if(trend) {
    e_tr <- effects$trend
    p_tr <- 1900 - 1970 + as.POSIXlt(harmon)$year + (yday365(harmon) - 1)/365
    p_tr <- e_tr$Estimate[which.min(abs(p_tr - e_tr$trend))]
  } else {
    p_tr <- 0
  }  

  p_sp <- if(spatial) e_sp$Estimate else 0
  p_we <- if(weekend) e_we else 0

  ## collect linear predictor
  p <- drop(p_tr + p_ha + p_sp + p_we)
  names(p) <- rownames(effects$param)
  
  ## transform to probabilities
  pr <- t(sapply(p, function(x) diff(c(0, pnorm(effects$cutoffs - x), 1))))
  rownames(pr) <- rownames(effects$param)
  colnames(pr) <- c("none", "low", "medium", "high")

  ## select subset of stations
  if(!is.null(stations)) pr <- pr[rownames(pr) %in% stations, , drop = FALSE]

  return(pr)
}
