#######################
## Code and packages ##
#######################

## packages and additional utilities
library("R2BayesX")
library("maptools")
library("zoo")
source("functions.R")


############################
## Data and fitted models ##
############################

## read shapefiles for Austria and save map
library("maptools")
austria <- readShapePoly(file.path("data", "shp", "at"),
  proj4string = CRS("+proj=longlat +datum=WGS84"))

## load rain data and meteorological station metainformation
if(!file.exists(file.path("data", "homstart.rda"))) source("homstart.R")
load(file.path("data", "homstart.rda"))

## load fitted model
if(!file.exists("homstart_effects.rda")) source("model.R")
load("homstart_effects.rda")


##############
## Graphics ##
##############

## plotting window
save_png <- TRUE
tspng <- function(file, height = 3, width = 6) {
  png(file, height = height, width = width,
    units = "in", res = 250, bg = "transparent")
}
mappng <- function(file, height = 4.5, width = 9) {
  png(file, height = height, width = width,
    units = "in", res = 250, bg = "transparent")
}

## color palettes
heat_redblue <- function(n) rev(heat_hcl(n, h = c(70, 240), l = c(95, 50), c = c(0, 60), power = 1))
diverge_yelblue <- function(n) diverge_hcl(n, h = c(240, 70), c = 70, l = c(40, 90), fixup = TRUE)

## time series plot: trend effect (across all stations)
if(save_png) tspng("homstart-effect-trend.png", height = 4, width = 5)
par(mar = c(4.1, 4.1, 0.1, 1.2))
plot(homstart_effects$trend, ylim = c(-0.5, 0.5), rug = FALSE,
  xlab = "Time", ylab = "Trend effect", axes = FALSE)
abline(h = 0, lty = 2)
axis(1, at = 10 * (-2:4), labels = 1970 + 10 * (-2:4))
axis(2)
box()
if(save_png) dev.off()

## time series plot: seasonal effect (per station)
if(save_png) tspng("homstart-effect-season.png", height = 4, width = 5)
par(mar = c(4.1, 4.1, 0.1, 1.2))
redblue1 <- hcl(c(260, 0, 0), c(80, 0, 80), 30, alpha = c(0.15, 0.1, 0.15))
redblue2 <- hcl(c(260, 0, 0, 0), c(80, 0, 80, 0), 30, alpha = c(0.3, 0.3, 0.3, 0))
latbreaks <- c(46.4, 46.7, 48.2, 48.8)
latcat <- cut(homstart_effects$param$lat, breaks = latbreaks)
matplot(x = homstart_effects$day[, 1L], y = homstart_effects$day[, -1L],
  type = "l", lty = 1, lwd = 2, col = redblue1[latcat],
  ylim = c(-0.5, 0.5), xlab = "Time", ylab = "Seasonal effect", axes = FALSE)
legend("topleft", c("Latitude", paste(latbreaks[3:1], latbreaks[4:2], sep = "-")),
  col = rev(redblue2), lty = 1, lwd = 2, bty = "n", cex = 0.8)
abline(h = 0, lty = 2)
Axis(as.Date(c("1970-01-01", "1970-12-31")), side = 1)
axis(2)
box()
if(save_png) dev.off()

## map: spatial effect
if(save_png) mappng("homstart-effect-spatial.png")
par(mar = rep(0.25, 4))
interpolmap(homstart_effects$spatial, map = austria,
  range = c(-0.5, 0.5), col = diverge_yelblue)
if(save_png) dev.off()

## map: spatial weekend effect
if(save_png) mappng("homstart-effect-weekend.png")
par(mar = rep(0.25, 4))
interpolmap(homstart_effects$param, select = "weekend", map = austria,
  range = c(-0.07, 0.07), col = diverge_yelblue, digits = 3)
if(save_png) dev.off()

## map: amplitude of seasonal effect
if(save_png) mappng("homstart-effect-amplitude.png")
par(mar = rep(0.25, 4))
interpolmap(homstart_effects$param, select = "amplitude1", map = austria,
  range = c(0, 0.5), symmetric = FALSE, col = heat_redblue)
if(save_png) dev.off()

