## Collect and read all required data:
##   - HOMSTART data
##     (need to be downloaded from http://www.zamg.ac.at/,
##      free for research purposes)
##   - metainformation about the corresponding meteoroligcal stations 
##     (provided in a .csv in our data.zip)

## read metainformation for meteorological stations
homstart <- read.csv2(file.path("data", "HOMSTART-stations.csv"))

## download and unzip HOMSTART data
if(!file.exists(file.path("data", "DATEN"))) {
  url <- "http://www.zamg.ac.at/docs/forschung/klimatologie/HOMSTART-Daten.zip"
  bsname <- basename(url)
  if(!file.exists(file.path("data", bsname))) download.file(url, file.path("data", bsname))
  unzip(file.path("data", bsname), exdir = "data")
}

## read in time series and save in .rda format
## first specify which series to read in
all_stations <- TRUE
include_rain <- TRUE
include_tempmin <- FALSE
include_tempmax <- FALSE

## which set of stations should be used
stations <- if(all_stations) homstart$id else homstart$id[homstart$elevation < 2000]
  
## convenience function to read in the homstart series
readhomstart <- function(f, which = 2, prefix = "s") {
  require("zoo")
  x <- read.zoo(f, header = FALSE, skip = 3, format = "%Y%m%d", na.string = "NA", drop = FALSE)
  colnames(x) <- paste(prefix, strsplit(f, "_", fixed = TRUE)[[1]][2], sep = "")
  x
}

## read data series
if(include_rain) {
  rain <- do.call("merge", lapply(Sys.glob("data/DATEN/Niederschlag/*.txt"), readhomstart))
  rain <- rain[, names(rain) %in% stations]
  stations <- names(rain)
}
if(include_tempmin) {
  tempmin <- do.call("merge", lapply(Sys.glob("data/DATEN/Tmin/*.txt"), readhomstart))
  tempmin <- tempmin[, names(tempmin) %in% stations]
  stations <- names(tempmin)
}
if(include_tempmax) {
  tempmax <- do.call("merge", lapply(Sys.glob("data/DATEN/Tmax/*.txt"), readhomstart))
  tempmax <- tempmax[, names(tempmax) %in% stations]
  stations <- names(tempmax)
}

## save all necessary data
homstart <- homstart[homstart$id %in% stations, ]
savelist <- c("homstart", "rain", "tempmin", "tempmax")[c(TRUE, include_rain, include_tempmin, include_tempmax)]
save(list = savelist, file = file.path("data", "homstart.rda"), compress = "bzip2")

## clean up global environment
remove(all_stations, bsname, homstart, 
  include_rain, include_tempmax, include_tempmin, rain,
  readhomstart, savelist, stations, url)

## clean up physical files (unzipped HOMSTART data)
unlink(file.path("data", "DATEN"), recursive = TRUE)
