## Fit STAR model to HOMSTART data
##   - Load data created in homstart.R
##   - preprocessing of full regressor matrix for use in R2BayesX
##   - call R2BayesX for model fitting

## load rain data and meteorological station metainformation
if(!file.exists(file.path("data", "homstart.rda"))) source("homstart.R")
load(file.path("data", "homstart.rda"))

## load packages and additional functions for data preprocessing
## (in particular tspp() which sets up the regressors)
library("R2BayesX")
library("zoo")
source("functions.R")

## setup model formula and estimate STAR model with R2BayesX
## (see the project site of R2BayesX for details on installation
## and estimation: http://bayesr.R-Forge.R-project.org/)
if(!file.exists("homstart_model.rda")) {

  ## generate the data sequentially for each station and save in plain text (.raw)
  ## (may take some time)
  if(!file.exists(file.path("data", "homstart_rain.raw"))) {
    for(i in names(rain)) {
      dat <- na.omit(tspp(rain[, i]))
      dat <- data.frame(
        cat = as.integer(dat$cat) - 1L,
        trend = dat$trend,
        long = homstart$long[homstart$id == i],
        lat = homstart$lat[homstart$id == i],
        model.matrix(~ 0 + (weekend + cos1 + cos2 + sin1 + sin2) %in% id,
          data = data.frame(
            id = factor(rep(rownames(homstart)[homstart$id == i],
              length = nrow(dat)),
              levels = rownames(homstart)),
            trend = dat$trend,
            as.data.frame(dat$harmon),
            weekend = as.integer(dat$weekend) - 1L
          )
        )
      )
      for(ch in c(".", ":", "_", "-"))
        names(dat) <- gsub(ch, "", names(dat), fixed = TRUE)
      append <- if(c(1:length(names(rain)))[names(rain) %in% i] < 2) FALSE else TRUE
      write.table(dat, file = file.path("data", "homstart_rain.raw"),
        quote = FALSE, row.names = FALSE, col.names = !append, append = append)
    }
  }

  ## set up formula
  f <- eval(parse(text = paste("c(\"", gsub(" ", "\", \"",
    readLines(file.path("data", "homstart_rain.raw"), n = 1L)), "\")", sep = "")))
  f <- grep("id", f, value = TRUE)
  f <- as.formula(paste("cat", "~", "sx(long, lat, bs = \"kr\", full = TRUE)", "+",
    "sx(trend)" , "+", paste(f[f != "id"], collapse = " + ")))

  ## fit the model
  homstart_model <- bayesx(f, family = "cumprobit", method = "REML", eps = 0.001,
    data = file.path(getwd(), "data", "homstart_rain.raw"))
  save(homstart_model, file = "homstart_model.rda")
} else {
  load("homstart_model.rda")
}

## extract and save fitted effects
homstart_effects <- extract_effects(homstart_model, homstart)
save(homstart_effects, file = "homstart_effects.rda", compress = "xz")

## clean up raw data file (if necessary)
if(file.exists(file.path("data", "homstart_rain.raw")))  file.remove(file.path("data", "homstart_rain.raw"))
